@@ -13,7 +13,7 @@
 #include <string.h>
 #include <slang.h>
 
-static char *Slsh_Version = "0.4-1";
+static char *Slsh_Version = "0.4-2";
 #define SLSHRC_FILE "slsh.rc"
 
 
@@ -261,6 +261,7 @@ static int load_startup_file (void)
    return 0;
 }
 
+#if 0
 static int is_script (char *file)
 {
    FILE *fp;
@@ -276,7 +277,7 @@ static int is_script (char *file)
    fclose (fp);
    return is;
 }
-
+#endif
 static void usage (void)
 {
    fprintf (stderr, "Usage: slsh [--help ] [ --version ] [-g] [FILENAME ...]\n");
@@ -336,13 +337,18 @@ int main (int argc, char **argv)
    else
      {
 	file = argv[1];
+#if 0
 	if (is_script (file))
 	  {
 	     argv++;
 	     argc--;
 	  }
+#else
+	argc--;
+	argv++;
+#endif
      }
-   fprintf (stdout, "slsh: argv[0]=%s\n", argv[0]);
+   /* fprintf (stdout, "slsh: argv[0]=%s\n", argv[0]); */
    if (-1 == SLang_set_argc_argv (argc, argv))
      return 1;
 
