@@ -2,7 +2,7 @@
 #define _PRIVATE_SLANG_H_
 /* header file for S-Lang internal structures that users do not (should not)
    need.  Use slang.h for that purpose. */
-/* Copyright (c) 1992, 1999, 2001 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -166,19 +166,19 @@ _SL_Object_Union_Type;
 
 typedef struct _SLang_Object_Type
 {
-   unsigned char data_type;	       /* SLANG_INT_TYPE, ... */
+   SLtype data_type;	       /* SLANG_INT_TYPE, ... */
    _SL_Object_Union_Type v;
 }
 SLang_Object_Type;
 
 struct _SLang_MMT_Type
 {
-   unsigned char data_type;	       /* int, string, etc... */
+   SLtype data_type;	       /* int, string, etc... */
    VOID_STAR user_data;	       /* address of user structure */
    unsigned int count;		       /* number of references */
 };
 
-extern int _SLang_pop_object_of_type (unsigned char, SLang_Object_Type *, int);
+extern int _SLang_pop_object_of_type (SLtype, SLang_Object_Type *, int);
 
 typedef struct
 {
@@ -203,9 +203,6 @@ extern int _SLstruct_init (void);
 extern int _SLstruct_define_struct (void);
 extern int _SLstruct_define_typedef (void);
 
-extern int _SLang_pop_datatype (unsigned char *);
-extern int _SLang_push_datatype (unsigned char);
-
 struct _SLang_Ref_Type
 {
    int is_global;
@@ -226,8 +223,8 @@ extern int _SL_decrement_frame_pointer (void);
 
 extern int SLang_pop(SLang_Object_Type *);
 extern void SLang_free_object (SLang_Object_Type *);
-extern int _SLanytype_typecast (unsigned char, VOID_STAR, unsigned int,
-				unsigned char, VOID_STAR);
+extern int _SLanytype_typecast (SLtype, VOID_STAR, unsigned int,
+				SLtype, VOID_STAR);
 extern void _SLstring_intrinsic (void);
 
 
@@ -323,10 +320,10 @@ extern int _SLusleep (unsigned long);
 /* frees upon error.  NULL __NOT__ ok. */
 extern int _SLang_push_slstring (char *);
 
-extern unsigned char _SLarith_promote_type (unsigned char);
-extern int _SLarith_get_precedence (unsigned char);
-extern int _SLarith_typecast (unsigned char, VOID_STAR, unsigned int,
-			      unsigned char, VOID_STAR);
+extern SLtype _SLarith_promote_type (SLtype);
+extern int _SLarith_get_precedence (SLtype);
+extern int _SLarith_typecast (SLtype, VOID_STAR, unsigned int,
+			      SLtype, VOID_STAR);
 
 extern int SLang_push(SLang_Object_Type *);
 extern int SLadd_global_variable (char *);
@@ -342,9 +339,11 @@ extern unsigned int _SLpc_convert_scancode (unsigned int, unsigned int, int);
 #define _SLTT_KEY_ALT		4
 #endif
 
-#ifdef REAL_UNIX_SYSTEM
-extern int SLtt_tigetflag (char *, char **);
-#endif
+typedef struct _SLterminfo_Type SLterminfo_Type;
+extern SLterminfo_Type *_SLtt_tigetent (char *);
+extern char *_SLtt_tigetstr (SLterminfo_Type *, char *);
+extern int _SLtt_tigetnum (SLterminfo_Type *, char *);
+extern int _SLtt_tigetflag (SLterminfo_Type *, char *);
 
 #if SLTT_HAS_NON_BCE_SUPPORT
 extern int _SLtt_get_bce_color_offset (void);
@@ -354,33 +353,33 @@ extern void (*_SLtt_color_changed_hook)(void);
 extern unsigned char SLang_Input_Buffer [SL_MAX_INPUT_BUFFER_LEN];
 
 extern int _SLregister_types (void);
-extern SLang_Class_Type *_SLclass_get_class (unsigned char);
+extern SLang_Class_Type *_SLclass_get_class (SLtype);
 extern VOID_STAR _SLclass_get_ptr_to_value (SLang_Class_Type *, SLang_Object_Type *);
-extern void _SLclass_type_mismatch_error (unsigned char, unsigned char);
+extern void _SLclass_type_mismatch_error (SLtype, SLtype);
 extern int _SLclass_init (void);
-extern int _SLclass_copy_class (unsigned char, unsigned char);
+extern int _SLclass_copy_class (SLtype, SLtype);
 
-extern unsigned char _SLclass_Class_Type [256];
+extern SLtype _SLclass_Class_Type [256];
 
-extern int (*_SLclass_get_typecast (unsigned char, unsigned char, int))
-(unsigned char, VOID_STAR, unsigned int,
- unsigned char, VOID_STAR);
+extern int (*_SLclass_get_typecast (SLtype, SLtype, int))
+(SLtype, VOID_STAR, unsigned int,
+ SLtype, VOID_STAR);
 
 extern int (*_SLclass_get_binary_fun (int, SLang_Class_Type *, SLang_Class_Type *, SLang_Class_Type **, int))
 (int,
- unsigned char, VOID_STAR, unsigned int,
- unsigned char, VOID_STAR, unsigned int,
+ SLtype, VOID_STAR, unsigned int,
+ SLtype, VOID_STAR, unsigned int,
  VOID_STAR);
 
 extern int (*_SLclass_get_unary_fun (int, SLang_Class_Type *, SLang_Class_Type **, int))
-(int, unsigned char, VOID_STAR, unsigned int, VOID_STAR);
+(int, SLtype, VOID_STAR, unsigned int, VOID_STAR);
 
 extern int _SLarith_register_types (void);
-extern unsigned char _SLarith_Arith_Types [];
-extern unsigned char _SLarith_Is_Arith_Type [256];
+extern SLtype _SLarith_Arith_Types [];
+extern SLtype _SLarith_Is_Arith_Type [256];
 extern int _SLarith_bin_op (SLang_Object_Type *, SLang_Object_Type *, int);
 
-extern int _SLarray_add_bin_op (unsigned char);
+extern int _SLarray_add_bin_op (SLtype);
 
 extern int _SLang_call_funptr (SLang_Name_Type *);
 extern void _SLset_double_format (char *);
@@ -405,8 +404,8 @@ extern int _SLarray_inline_array (void);
 extern int _SLarray_wildcard_array (void);
 
 extern int
-_SLarray_typecast (unsigned char, VOID_STAR, unsigned int,
-		   unsigned char, VOID_STAR, int);
+_SLarray_typecast (SLtype, VOID_STAR, unsigned int,
+		   SLtype, VOID_STAR, int);
 
 extern int _SLarray_aput_transfer_elem (SLang_Array_Type *, int *,
 					VOID_STAR, unsigned int, int);
@@ -415,16 +414,16 @@ extern int _SLarray_aget_transfer_elem (SLang_Array_Type *, int *,
 extern void _SLarray_free_array_elements (SLang_Class_Type *, VOID_STAR, unsigned int);
 
 extern SLang_Foreach_Context_Type *
-_SLarray_cl_foreach_open (unsigned char, unsigned int);
-extern void _SLarray_cl_foreach_close (unsigned char, SLang_Foreach_Context_Type *);
-extern int _SLarray_cl_foreach (unsigned char, SLang_Foreach_Context_Type *);
+_SLarray_cl_foreach_open (SLtype, unsigned int);
+extern void _SLarray_cl_foreach_close (SLtype, SLang_Foreach_Context_Type *);
+extern int _SLarray_cl_foreach (SLtype, SLang_Foreach_Context_Type *);
 
 extern int _SLarray_matrix_multiply (void);
 extern void (*_SLang_Matrix_Multiply)(void);
 
 extern int _SLarray_init_slarray (void);
 extern SLang_Array_Type *
-SLang_create_array1 (unsigned char, int, VOID_STAR, int *, unsigned int, int);
+SLang_create_array1 (SLtype, int, VOID_STAR, int *, unsigned int, int);
 
 extern int _SLcompile_push_context (SLang_Load_Type *);
 extern int _SLcompile_pop_context (void);
@@ -444,7 +443,7 @@ typedef struct
 #if _SLANG_HAS_DEBUG_CODE
    int line_number;
 #endif
-   unsigned char type;
+   SLtype type;
 }
 _SLang_Token_Type;
 
@@ -459,6 +458,7 @@ extern void (*_SLcompile_ptr)(_SLang_Token_Type *);
  */
 
 /* Special tokens */
+#define ILLEGAL_TOKEN	0x00	       /* no token has this value */
 #define EOF_TOKEN	0x01
 #define RPN_TOKEN	0x02
 #define NL_TOKEN	0x03
@@ -492,8 +492,8 @@ extern void (*_SLcompile_ptr)(_SLang_Token_Type *);
 #define IDENT_TOKEN	0x20
 #define ARRAY_TOKEN	0x21
 #define DOT_TOKEN	0x22
-#define IS_LVALUE_TOKEN (((t) <= DOT_TOKEN) && ((t) >= IDENT_TOKEN))
-
+#define METHOD_TOKEN	0x24
+#define IS_LVALUE_TOKEN (((t) <= METHOD_TOKEN) && ((t) >= IDENT_TOKEN))
 /* do not use these values */
 #define RESERVED4_TOKEN	0x23	       /* # */
 #define RESERVED5_TOKEN 0x25	       /* % */
