@@ -1,5 +1,5 @@
 /* Array manipulation routines for S-Lang */
-/* Copyright (c) 1997, 1999, 2001 John E. Davis
+/* Copyright (c) 1997, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -8,7 +8,7 @@
 
 #include "slinclud.h"
 
-#define SL_APP_WANTS_FOREACH
+/* #define SL_APP_WANTS_FOREACH */
 #include "slang.h"
 #include "_slang.h"
 
@@ -223,7 +223,6 @@ static int do_method_for_all_elements (SLang_Array_Type *at,
 
 void SLang_free_array (SLang_Array_Type *at)
 {
-   VOID_STAR data;
    unsigned int flags;
 
    if (at == NULL) return;
@@ -234,7 +233,6 @@ void SLang_free_array (SLang_Array_Type *at)
 	return;
      }
 
-   data = at->data;
    flags = at->flags;
 
    if (flags & SLARR_DATA_VALUE_IS_INTRINSIC)
@@ -243,7 +241,11 @@ void SLang_free_array (SLang_Array_Type *at)
    if (flags & SLARR_DATA_VALUE_IS_POINTER)
      (void) do_method_for_all_elements (at, destroy_element, NULL);
 
-   SLfree ((char *) data);
+   if (at->free_fun != NULL)
+     at->free_fun (at);
+   else
+     SLfree ((char *) at->data);
+
    SLfree ((char *) at);
 }
 
@@ -447,7 +449,7 @@ static int push_create_new_array (void)
 
    num_dims = (SLang_Num_Function_Args - 1);
 
-   if (-1 == _SLang_pop_datatype (&type))
+   if (-1 == SLang_pop_datatype (&type))
      return -1;
 
    anew = (_SLclass_get_class (type))->cl_anew;
@@ -787,6 +789,23 @@ convert_nasty_index_objs (SLang_Array_Type *at,
 		   * 
 		   * However, [-1:-3] does not refer to any of the elements.
 		   */
+		  
+		  /* FIXME: Priority=High; I think this is broken behavior
+		   * and should be rethought.  That is, a[[0:-1]] should 
+		   * specify no elements.   That is, the behavior should be:
+		   * [0:9]     ==> first 9 elements
+		   * [-3:-1]   ==> last 3 elements
+		   * [0:-1]    ==> [] (no elements)
+		   * [0:-1:-1] ==> [0, -1] ==> first and last elements
+		   * [-1:-3]  ==> []
+		   * 
+		   * Unfortunately, this is going to be difficult to fix
+		   * because of the way rubber ranges are stored:
+		   *   [*] ==> [0:-1]
+		   *
+		   * Perhaps it is just best to document this behavior.
+		   * Sigh.
+		   */
 		  if ((first_index = r->first_index) < 0)
 		    {
 		       if (at_dims_i != 0)
@@ -1685,7 +1704,7 @@ static void array_info (void)
 	if (0 == SLang_push_array (bt, 1))
 	  {
 	     (void) SLang_push_integer ((int) at->num_dims);
-	     (void) _SLang_push_datatype (at->data_type);
+	     (void) SLang_push_datatype (at->data_type);
 	  }
      }
 
@@ -2076,55 +2095,26 @@ int _SLarray_inline_array (void)
 	     type = SLANG_ARRAY_TYPE;
 	     break;
 
-	   case SLANG_INT_TYPE:
-	     switch (type)
-	       {
-#if SLANG_HAS_FLOAT
-		case SLANG_DOUBLE_TYPE:
-		  break;
-#endif
-#if SLANG_HAS_COMPLEX
-		case SLANG_COMPLEX_TYPE:
-		  break;
-#endif
-		default:
-		  goto type_mismatch;
-	       }
-	     break;
-#if SLANG_HAS_FLOAT
-	   case SLANG_DOUBLE_TYPE:
-	     switch (type)
-	       {
-		case SLANG_INT_TYPE:
-		  type = SLANG_DOUBLE_TYPE;
-		  break;
-# if SLANG_HAS_COMPLEX
-		case SLANG_COMPLEX_TYPE:
-		  break;
-# endif
-		default:
-		  goto type_mismatch;
-	       }
-	     break;
-#endif
-#if SLANG_HAS_COMPLEX
 	   case SLANG_COMPLEX_TYPE:
-	     switch (type)
-	       {
-		case SLANG_INT_TYPE:
-		case SLANG_DOUBLE_TYPE:
-		  type = SLANG_COMPLEX_TYPE;
-		  break;
-
-		default:
-		  goto type_mismatch;
-	       }
+	     if (0 == _SLarith_Is_Arith_Type[type])
+	       goto type_mismatch;
+	     
+	     type = this_type;
 	     break;
-#endif
+	     
 	   default:
-	     type_mismatch:
-	     _SLclass_type_mismatch_error (type, this_type);
-	     return -1;
+	     if (0 == _SLarith_Is_Arith_Type[this_type])
+	       goto type_mismatch;
+	     
+	     if (type == SLANG_COMPLEX_TYPE)
+	       break;
+	     
+	     if (0 == _SLarith_Is_Arith_Type[type])
+	       goto type_mismatch;
+	     
+	     if (_SLarith_get_precedence (this_type) > _SLarith_get_precedence (type))
+	       type = this_type;
+	     break;
 	  }
 	count--;
      }
@@ -2171,6 +2161,10 @@ int _SLarray_inline_array (void)
      }
 
    return SLang_push_array (at, 1);
+
+   type_mismatch:
+   _SLclass_type_mismatch_error (type, this_type);
+   return -1;
 }
 
 static int array_binary_op_result (int op, unsigned char a, unsigned char b,
@@ -2526,7 +2520,7 @@ static void array_map (void)
 
    num_elements = args[i_control].at->num_elements;
 
-   if (-1 == _SLang_pop_datatype (&type))
+   if (-1 == SLang_pop_datatype (&type))
      goto return_error;
 
    if (type == SLANG_UNDEFINED_TYPE)   /* Void_Type */
@@ -2548,6 +2542,9 @@ static void array_map (void)
 	  args[i].increment = ati->sizeof_type;
 	/* memset already guarantees increment to be zero */
 
+	/* FIXME: array_map on an empty array should return an empty array
+	 * and not generate an error.
+	 */
 	if (ati->num_elements == 0)
 	  {
 	     SLang_verror (0, "array_map: function argument %d of %d is an empty array", 
@@ -2960,7 +2957,7 @@ array_datatype_deref (unsigned char type)
 	goto return_error;
      }
 
-   if (-1 == _SLang_pop_datatype (&type))
+   if (-1 == SLang_pop_datatype (&type))
      goto return_error;
 
    if (NULL == (at = SLang_create_array (type, 0, NULL,
@@ -3060,7 +3057,7 @@ struct _SLang_Foreach_Context_Type
 };
 
 SLang_Foreach_Context_Type *
-_SLarray_cl_foreach_open (unsigned char type, unsigned int num)
+_SLarray_cl_foreach_open (SLtype type, unsigned int num)
 {
    SLang_Foreach_Context_Type *c;
 
@@ -3087,7 +3084,7 @@ _SLarray_cl_foreach_open (unsigned char type, unsigned int num)
    return c;
 }
 
-void _SLarray_cl_foreach_close (unsigned char type, SLang_Foreach_Context_Type *c)
+void _SLarray_cl_foreach_close (SLtype type, SLang_Foreach_Context_Type *c)
 {
    (void) type;
    if (c == NULL) return;
@@ -3095,7 +3092,7 @@ void _SLarray_cl_foreach_close (unsigned char type, SLang_Foreach_Context_Type *
    SLfree ((char *) c);
 }
 
-int _SLarray_cl_foreach (unsigned char type, SLang_Foreach_Context_Type *c)
+int _SLarray_cl_foreach (SLtype type, SLang_Foreach_Context_Type *c)
 {
    SLang_Array_Type *at;
    VOID_STAR data;
