@@ -1,4 +1,4 @@
-/* Copyright (c) 1998, 1999, 2001 John E. Davis
+/* Copyright (c) 1998, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -6,7 +6,7 @@
  */
 #include "slinclud.h"
 
-#define SL_APP_WANTS_FOREACH
+/* #define SL_APP_WANTS_FOREACH */
 #include "slang.h"
 #include "_slang.h"
 
@@ -278,8 +278,8 @@ static int assoc_aget (unsigned char type, unsigned int num_indices)
 #if _SLANG_OPTIMIZE_FOR_SPEED
 	if (SLANG_CLASS_TYPE_SCALAR == _SLclass_Class_Type[obj->data_type])
 	  ret = SLang_push (obj);
-#endif
 	else
+#endif
 	  ret = _SLpush_slang_obj (obj);
      }
 
@@ -346,7 +346,7 @@ static int assoc_anew (unsigned char type, unsigned int num_dims)
 	has_default_value = 1;
 	/* drop */
       case 1:
-	if (0 == _SLang_pop_datatype (&type))
+	if (0 == SLang_pop_datatype (&type))
 	  break;
 	num_dims--;
 	/* drop */
@@ -450,7 +450,7 @@ static void assoc_get_values (SLang_Assoc_Array_Type *a)
 {
    SLang_Array_Type *at;
    int num;
-   unsigned int i, j;
+   unsigned int j;
    char *dest_data;
    unsigned char type;
    SLang_Class_Type *cl;
@@ -471,7 +471,6 @@ static void assoc_get_values (SLang_Assoc_Array_Type *a)
 
    dest_data = (char *)at->data;
 
-   i = 0;
    for (j = 0; j < SLASSOC_HASH_TABLE_SIZE; j++)
      {
 	_SLAssoc_Array_Element_Type *e;
@@ -487,7 +486,6 @@ static void assoc_get_values (SLang_Assoc_Array_Type *a)
 
 	     dest_data += sizeof_type;
 	     e = e->next;
-	     i++;
 	  }
      }
    (void) SLang_push_array (at, 1);
