@@ -1,5 +1,5 @@
 /* User defined objects */
-/* Copyright (c) 1992, 1999, 2001 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -7,6 +7,7 @@
  */
 #include "slinclud.h"
 
+/* #define SL_APP_WANTS_FOREACH */
 #include "slang.h"
 #include "_slang.h"
 
@@ -215,6 +216,18 @@ static int default_acopy (unsigned char type, VOID_STAR from, VOID_STAR to)
    return (*cl->cl_apop) (type, to);
 }
 
+static int scalar_acopy (SLtype type, VOID_STAR from, VOID_STAR to)
+{
+   memcpy ((char *)to, (char *)from, _SLclass_get_class (type)->cl_sizeof_type);
+   return 0;
+}
+
+int SLclass_dup_object (SLtype type, VOID_STAR from, VOID_STAR to)
+{
+   SLang_Class_Type *cl = _SLclass_get_class (type);
+   return cl->cl_acopy (type, from, to);
+}
+
 static int default_dereference_object (unsigned char type, VOID_STAR ptr)
 {
    (void) ptr;
@@ -414,7 +427,7 @@ SLang_Class_Type *SLclass_allocate_class (char *name)
 
 static int DataType_Ids [256];
 
-int _SLang_push_datatype (unsigned char data_type)
+int SLang_push_datatype (unsigned char data_type)
 {
    /* This data type could be a copy of another type, e.g., short and
     * int if they are the same size (Int16 == Short).  So, make sure
@@ -447,10 +460,10 @@ static int datatype_deref (unsigned char type, VOID_STAR ptr)
 static int datatype_push (unsigned char type_unused, VOID_STAR ptr)
 {
    (void) type_unused;
-   return _SLang_push_datatype (*(int *) ptr);
+   return SLang_push_datatype (*(int *) ptr);
 }
 
-int _SLang_pop_datatype (unsigned char *type)
+int SLang_pop_datatype (unsigned char *type)
 {
    int i;
 
@@ -463,7 +476,7 @@ int _SLang_pop_datatype (unsigned char *type)
 
 static int datatype_pop (unsigned char type, VOID_STAR ptr)
 {
-   if (-1 == _SLang_pop_datatype (&type))
+   if (-1 == SLang_pop_datatype (&type))
      return -1;
 
    *(int *) ptr = type;
@@ -578,6 +591,7 @@ int SLclass_register_class (SLang_Class_Type *cl, unsigned char type, unsigned i
 	  return method_undefined_error (type, "pop", name);
 	if (cl->cl_fread == NULL) cl->cl_fread = scalar_fread;
 	if (cl->cl_fwrite == NULL) cl->cl_fwrite = scalar_fwrite;
+	if (cl->cl_acopy == NULL) cl->cl_acopy = scalar_acopy;
 
 	can_binop = 1;
 	break;
@@ -598,6 +612,7 @@ int SLclass_register_class (SLang_Class_Type *cl, unsigned char type, unsigned i
 	cl->cl_adestroy = default_destroy_simple;
 	if (cl->cl_fread == NULL) cl->cl_fread = scalar_fread;
 	if (cl->cl_fwrite == NULL) cl->cl_fwrite = scalar_fwrite;
+	if (cl->cl_acopy == NULL) cl->cl_acopy = scalar_acopy;
 	can_binop = 1;
 	break;
 
@@ -782,6 +797,14 @@ int SLclass_set_push_function (SLang_Class_Type *cl, int (*f)(unsigned char, VOI
    return 0;
 }
 
+int SLclass_set_acopy_function (SLang_Class_Type *cl, int (*f)(SLtype, VOID_STAR, VOID_STAR))
+{
+   if (cl == NULL) return -1;
+   cl->cl_acopy = f;
+
+   return 0;
+}
+
 int SLclass_set_string_function (SLang_Class_Type *cl, char *(*f)(unsigned char, VOID_STAR))
 {
    if (cl == NULL) return -1;
