@@ -1,5 +1,5 @@
 /* cmd line facility for slang */
-/* Copyright (c) 1992, 1999, 2001 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -131,7 +131,7 @@ static int allocate_arg_space (SLcmd_Cmd_Table_Type *table, int argc, unsigned i
 
 int SLcmd_execute_string (char *str, SLcmd_Cmd_Table_Type *table)
 {
-   char *s, *b = NULL, *arg_type, *last_str, *cmd_name;
+   char *s, *arg_type, *last_str, *cmd_name;
    SLcmd_Cmd_Type *cmd;
    char *buf;
    int token_present;
@@ -200,7 +200,7 @@ int SLcmd_execute_string (char *str, SLcmd_Cmd_Table_Type *table)
 
 	if (token_present)
 	  {
-	     b = buf;
+	     char *b = buf;
 	     len = strlen (b);
 
 	     if ((*b == '"') && (len > 1))
