@@ -1,4 +1,4 @@
-/* Copyright (c) 1992, 1999 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -26,7 +26,7 @@
  * and MSC don't, so we can check for those using ifdef __BORLAND_V5__
  */
 
-#if defined(__BORLANDC__) && (__BORLANDC__>>8 == 0x5)
+#if defined(__BORLANDC__) && ((__BORLANDC__>>8) == 0x5)
 # define __BORLAND_V5__
 #endif
 
@@ -101,7 +101,7 @@
 # define __16_BIT_SYSTEM__	1
 #endif
 
-#if defined(__WATCOMC__)
+#if defined(__WATCOMC__) && !defined(__QNX__)
 # undef unix
 # undef __unix__
 #endif /* __WATCOMC__ */
@@ -140,11 +140,11 @@
 #endif
 
 /* Does VMS have this??? */
-#if !defined(VMS) && !defined(_MSC_VER) && !defined(__WATCOMC__)
+#if !defined(VMS) && !defined(_MSC_VER) && (!defined(__WATCOMC__) || defined(__QNX__))
 # define HAVE_DIRENT_H	1
 #endif
 
-#if defined(VMS) || defined(__WATCOMC__)
+#if defined(VMS) || (defined(__WATCOMC__) && !defined(__QNX__))
 # define HAVE_DIRECT_H 1
 #endif
 
