@@ -1,4 +1,4 @@
-/* Copyright (c) 1998, 1999, 2001 John E. Davis
+/* Copyright (c) 1998, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -107,6 +107,62 @@ int SLcurses_nodelay (SLcurses_Window_Type *w, int onoff)
    return 0;
 }
 
+static unsigned char Keyboard_Buffer[256];
+static unsigned char *Keyboard_Buffer_Start = Keyboard_Buffer;
+static unsigned char *Keyboard_Buffer_Stop = Keyboard_Buffer;
+
+static int getkey_function (void)
+{
+   int ch;
+
+   ch = SLang_getkey ();
+   if (ch != SLANG_GETKEY_ERROR)
+     {
+	*Keyboard_Buffer_Stop++ = (unsigned char) ch;
+	if (Keyboard_Buffer_Stop == Keyboard_Buffer + sizeof (Keyboard_Buffer))
+	  Keyboard_Buffer_Stop = Keyboard_Buffer;
+     }
+   
+   return ch;
+}
+
+static int get_buffered_key (void)
+{
+   int ch;
+   
+   if (Keyboard_Buffer_Stop == Keyboard_Buffer_Start)
+     return SLANG_GETKEY_ERROR;
+
+   ch = *Keyboard_Buffer_Start++;
+   if (Keyboard_Buffer_Start == Keyboard_Buffer + sizeof (Keyboard_Buffer))
+     Keyboard_Buffer_Start = Keyboard_Buffer;
+
+   return ch;
+}
+
+static int get_keypad_key (void)
+{
+   int ch;
+   
+   if (Keyboard_Buffer_Stop != Keyboard_Buffer_Start)
+     return get_buffered_key ();
+
+   ch = SLang_getkey ();
+   if (ch == '\033')
+     {
+	if (0 == SLang_input_pending (ESCDELAY / 100))
+	  return ch;
+     }
+   else if (ch == SLANG_GETKEY_ERROR) return ERR;
+   SLang_ungetkey (ch);
+   ch = SLkp_getkey ();
+   if (ch == SL_KEY_ERR)
+     ch = get_buffered_key ();
+   else
+     Keyboard_Buffer_Stop = Keyboard_Buffer_Start;
+   return ch;
+}
+
 int SLcurses_wgetch (SLcurses_Window_Type *w)
 {
    if (w == NULL)
@@ -114,21 +170,13 @@ int SLcurses_wgetch (SLcurses_Window_Type *w)
 
    SLcurses_wrefresh (w);
 
-   if ((w->delay_off == -1) ||
-       SLang_input_pending (w->delay_off))
+   if ((Keyboard_Buffer_Start != Keyboard_Buffer_Stop)
+       || (w->delay_off == -1)
+       || SLang_input_pending (w->delay_off))
      {
 	if (w->use_keypad)
-	  {
-	     int ch = SLang_getkey ();
-	     if (ch == '\033')
-	       {
-		  if (0 == SLang_input_pending (ESCDELAY / 100))
-		    return ch;
-	       }
-	     else if (ch == 0xFFFF) return ERR;
-	     SLang_ungetkey (ch);
-	     return SLkp_getkey ();
-	  }
+	  return get_keypad_key ();
+
 	return SLang_getkey ();
      }
 
@@ -267,6 +315,7 @@ SLcurses_Window_Type *SLcurses_initscr (void)
 	SLang_exit_error ("SLcurses_initscr: init failed\n");
 	return NULL;
      }
+   SLkp_set_getkey_function (getkey_function);
 
 #ifdef SIGINT
    signal (SIGINT, sigint_handler);
