@@ -1,4 +1,4 @@
-/* Copyright (c) 1992, 1999, 2001 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -209,7 +209,7 @@ static char *Disable_Status_line_Str;  /* ds */
 static char *Return_From_Status_Line_Str;   /* fs */
 static char *Goto_Status_Line_Str;     /* ts */
 static int Num_Status_Line_Columns;    /* ws */
-static int Status_Line_Esc_Ok;	       /* es */
+/* static int Status_Line_Esc_Ok;	 */       /* es */
 
 /* static int Len_Curs_F_Str = 5; */
 
@@ -1255,6 +1255,9 @@ void SLtt_set_alt_char_set (int i)
 {
    static int last_i;
    if (SLtt_Has_Alt_Charset == 0) return;
+
+   i = (i != 0);
+
    if (i == last_i) return;
    tt_write_string (i ? Start_Alt_Chars_Str : End_Alt_Chars_Str );
    last_i = i;
@@ -1486,7 +1489,10 @@ static void send_attr_str (SLsmg_Char_Type *s)
 		      /* it is a space so only consider it different if it
 		       * has different attributes.
 		       */
-		      (attr & BGALL_MASK) != (Current_Fgbg & BGALL_MASK))
+		      (attr != Current_Fgbg))
+		    /* The previous line was: */
+		    /* (attr & BGALL_MASK) != (Current_Fgbg & BGALL_MASK)) */
+		    /* However, it does not account for ACS */
 		    {
 		       if (p != out)
 			 {
@@ -1954,76 +1960,102 @@ static void get_color_info (void)
 
 #ifdef __unix__
 
-# ifndef USE_TERMCAP
-static char *Tbuf;
-static char *Tstr_Buf;
-
-#  define tgetstr SLtt_tigetstr
-#  define tgetent SLtt_tigetent
-#  define TGETNUM(x) SLtt_tigetnum((x), &Tbuf)
-#  define TGETFLAG(x) (SLtt_tigetflag((x), &Tbuf) > 0)
-# else
+static int Termcap_Initalized = 0;
 
+#ifdef USE_TERMCAP
+/* Termcap based system */
+static char Termcap_Buf[4096];
+static char Termcap_String_Buf[4096];
+static char *Termcap_String_Ptr;
 extern char *tgetstr(char *, char **);
 extern int tgetent(char *, char *);
 extern int tgetnum(char *);
 extern int tgetflag(char *);
-static char Tstr_Buf[1024];
-static char Tbuf[4096];
-#  define TGETNUM tgetnum
-#  define TGETFLAG tgetflag
-# endif
+#else
+/* Terminfo */
+static SLterminfo_Type *Terminfo;
+#endif
 
-static char *my_tgetstr(char *what, char **p)
+#define TGETFLAG(x) (SLtt_tgetflag(x) > 0)
+
+static char *fixup_tgetstr (char *what)
 {
    register char *w, *w1;
    char *wsave;
-   what = tgetstr(what, p);
-   if (what != NULL)
+   
+   if (what == NULL)
+     return NULL;
+
+   /* Check for AIX brain-damage */
+   if (*what == '@')
+     return NULL;
+
+   /* lose pad info --- with today's technology, term is a loser if
+    it is really needed */
+   while ((*what == '.') ||
+	  ((*what >= '0') && (*what <= '9'))) what++;
+   if (*what == '*') what++;
+   
+   /* lose terminfo padding--- looks like $<...> */
+   w = what;
+   while (*w) if ((*w++ == '$') && (*w == '<'))
      {
-	/* Check for AIX brain-damage */
-	if (*what == '@')
-	  return NULL;
-
-	/* lose pad info --- with today's technology, term is a loser if
-	   it is really needed */
-	while ((*what == '.') ||
-	       ((*what >= '0') && (*what <= '9'))) what++;
-	if (*what == '*') what++;
-
-	/* lose terminfo padding--- looks like $<...> */
-        w = what;
-	while (*w) if ((*w++ == '$') && (*w == '<'))
-	  {
-	     w1 = w - 1;
-	     while (*w && (*w != '>')) w++;
-	     if (*w == 0) break;
-	     w++;
-	     wsave = w1;
-	     while ((*w1++ = *w++) != 0);
-	     w = wsave;
-	  }
-	if (*what == 0) what = NULL;
+	w1 = w - 1;
+	while (*w && (*w != '>')) w++;
+	if (*w == 0) break;
+	w++;
+	wsave = w1;
+	while ((*w1++ = *w++) != 0);
+	w = wsave;
      }
-   return(what);
+
+   if (*what == 0) what = NULL;
+   return what;
 }
 
-char *SLtt_tgetstr (char *s)
+char *SLtt_tgetstr (char *cap)
 {
-# ifdef USE_TERMCAP
-   static
-# endif
-     char *p = Tstr_Buf;
-   return my_tgetstr (s, &p);
+   char *s;
+
+   if (Termcap_Initalized == 0)
+     return NULL;
+   
+#ifdef USE_TERMCAP
+   s = tgetstr (cap, &Termcap_String_Ptr);
+#else
+   s = _SLtt_tigetstr (Terminfo, cap);
+#endif
+
+   /* Do not strip pad info for alternate character set.  I need to make
+    * this more general.
+    */
+   /* FIXME: Priority=low; */
+   if (0 == strcmp (cap, "ac"))
+     return s;
+
+   return fixup_tgetstr (s);
 }
 
 int SLtt_tgetnum (char *s)
 {
-   return TGETNUM (s);
+   if (Termcap_Initalized == 0)
+     return -1;
+#ifdef USE_TERMCAP
+   return tgetnum (s);
+#else
+   return _SLtt_tigetnum (Terminfo, s);
+#endif
 }
+
 int SLtt_tgetflag (char *s)
 {
-   return TGETFLAG (s);
+   if (Termcap_Initalized == 0)
+     return -1;
+#ifdef USE_TERMCAP
+   return tgetflag (s);
+#else
+   return _SLtt_tigetflag (Terminfo, s);
+#endif
 }
 
 static int Vt100_Like = 0;
@@ -2100,7 +2132,7 @@ int SLtt_initialize (char *term)
 		   || !strcmp (term, "screen"));
 
 # ifndef USE_TERMCAP
-   if (NULL == (Tbuf = tgetent (term)))
+   if (NULL == (Terminfo = _SLtt_tigetent (term)))
      {
 	if (almost_vtxxx) /* Special cases. */
 	  {
@@ -2108,16 +2140,19 @@ int SLtt_initialize (char *term)
 	     if (!strcmp (term, "vt100")) vt102 = 0;
 	     get_color_info ();
    	     SLtt_set_term_vtxxx (&vt102);
+	     (void) SLtt_get_screen_size ();
 	     return 0;
 	  }
 	return -1;
      }
-   Tstr_Buf = Tbuf;
 # else				       /* USE_TERMCAP */
-   if (1 != tgetent(Tbuf, term))
+   if (1 != tgetent(Termcap_Buf, term))
      return -1;
+   Termcap_String_Ptr = Termcap_String_Buf;
 # endif				       /* NOT USE_TERMCAP */
 
+   Termcap_Initalized = 1;
+
    Cls_Str = SLtt_tgetstr ("cl");
    Curs_Pos_Str = SLtt_tgetstr ("cm");
 
@@ -2239,8 +2274,8 @@ int SLtt_initialize (char *term)
 	Disable_Status_line_Str = SLtt_tgetstr ("ds");
 	Return_From_Status_Line_Str = SLtt_tgetstr ("fs");
 	Goto_Status_Line_Str = SLtt_tgetstr ("ts");
-	Status_Line_Esc_Ok = TGETFLAG("es");
-	Num_Status_Line_Columns = TGETNUM("ws");
+	/* Status_Line_Esc_Ok = TGETFLAG("es"); */
+	Num_Status_Line_Columns = SLtt_tgetnum ("ws");
 	if (Num_Status_Line_Columns < 0) Num_Status_Line_Columns = 0;
      }
 
@@ -2272,7 +2307,7 @@ int SLtt_initialize (char *term)
 
    if (Worthless_Highlight == 0)
      {				       /* Magic cookie glitch */
-	Worthless_Highlight = (TGETNUM ("sg") > 0);
+	Worthless_Highlight = (SLtt_tgetnum ("sg") > 0);
      }
 
    if (Worthless_Highlight)
@@ -2287,7 +2322,7 @@ int SLtt_initialize (char *term)
 	Color_Bg_Str = SLtt_tgetstr ("Sb");   /* setb */
      }
 
-   if (-1 == (Max_Terminfo_Colors = SLtt_tgetnum ("Co")))
+   if ((Max_Terminfo_Colors = SLtt_tgetnum ("Co")) < 0)
      Max_Terminfo_Colors = 8;
 
    if ((Color_Bg_Str != NULL) && (Color_Fg_Str != NULL))
