@@ -1,4 +1,4 @@
-/* Copyright (c) 1998, 1999 John E. Davis
+/* Copyright (c) 1998, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -26,6 +26,9 @@
 
 static char *Module_Path;
 #define MODULE_PATH_ENV_NAME	"SLANG_MODULE_PATH"
+#ifndef MODULE_INSTALL_DIR
+# define MODULE_INSTALL_DIR "/usr/local/lib/slang/modules"
+#endif
 
 typedef struct _Handle_Type
 {
@@ -90,13 +93,17 @@ static Handle_Type *find_handle (char *name)
 
 static int import_from_library (char *name, 
 				char *init_fun_name, char *deinit_fun_name,
-				char *file)
+				char *file,
+				char *ns,
+				char *ns_init_fun_name)
 {
    VOID_STAR handle;
    int (*init_fun) (void);
+   int (*ns_init_fun) (char *);
    void (*deinit_fun) (void);
    char *err;
    char filebuf[1024];
+   char *fun_name;
 
    if (NULL != find_handle (name))
      return 0;			       /* already loaded */
@@ -122,6 +129,9 @@ static int import_from_library (char *name,
 	     continue;
 	  }
 
+	/* Purify reports that dlerror returns a pointer that generates UMR
+	 * errors.  There is nothing that I can do about that....
+	 */
 	if (NULL == (err = (char *) dlerror ()))
 	  err = "UNKNOWN";
 
@@ -130,43 +140,76 @@ static int import_from_library (char *name,
 	return -1;
      }
 
-   init_fun = (int (*)(void)) dlsym (handle, init_fun_name);
-
-   if (init_fun == NULL)
+   fun_name = ns_init_fun_name;
+   ns_init_fun = (int (*)(char *)) dlsym (handle, fun_name);
+   if (ns_init_fun == NULL) 
+     {
+	if ((ns != NULL) 
+	    && (0 != strcmp (ns, "Global")))
+	  goto return_error;
+	
+	fun_name = init_fun_name;
+	init_fun = (int (*)(void)) dlsym (handle, fun_name);
+	if (init_fun == NULL)
+	  goto return_error;
+	
+	if (-1 == (*init_fun) ())
+	  {
+	     dlclose (handle);
+	     return -1;
+	  }
+     }
+   else if (-1 == (*ns_init_fun) (ns))
      {
-	if (NULL == (err = (char *) dlerror ()))
-	  err = "UNKNOWN";
-
 	dlclose (handle);
-	SLang_verror (SL_INTRINSIC_ERROR,
-		      "Unable to get symbol %s from %s: %s",
-		      name, file, err);
 	return -1;
      }
 
+
    deinit_fun = (void (*)(void)) dlsym (handle, deinit_fun_name);
 
-   if (-1 == (*init_fun) ())
-     {
-	dlclose (handle);
-	return -1;
-     }
-
    (void) save_handle (name, handle, deinit_fun);
    return 0;
+
+   return_error:
+   
+   if (NULL == (err = (char *) dlerror ()))
+     err = "UNKNOWN";
+
+   dlclose (handle);
+   SLang_verror (SL_INTRINSIC_ERROR,
+		 "Unable to get symbol %s from %s: %s",
+		 name, file, err);
+   return -1;
 }
 
-static void import_module (char *module)
+static void import_module (void)
 {
    char module_name[256];
    char symbol_name[256];
    char deinit_name[256];
+   char ns_init_name[256];
    char *path;
    char *file;
+   char *module;
+   char *ns = NULL;
+
+   if (SLang_Num_Function_Args == 2)
+     {
+	if (-1 == SLang_pop_slstring (&ns))
+	  return;	
+     }
+   
+   if (-1 == SLang_pop_slstring (&module))
+     {
+	SLang_free_slstring (ns);      /* NULL ok */
+	return;
+     }
 
    _SLsnprintf (symbol_name, sizeof(symbol_name), "init_%s_module", module);
    _SLsnprintf (module_name, sizeof(module_name), "%s-module.so", module);
    _SLsnprintf (deinit_name, sizeof(deinit_name), "deinit_%s_module", module);
+   _SLsnprintf (ns_init_name, sizeof (ns_init_name), "init_%s_module_ns", module);
 
    if (Module_Path != NULL)
      file = SLpath_find_file_in_path (Module_Path, module_name);
@@ -176,15 +219,18 @@ static void import_module (char *module)
        && (NULL != (path = getenv (MODULE_PATH_ENV_NAME))))
      file = SLpath_find_file_in_path (path, module_name);
 
+   if (file == NULL)
+     file = SLpath_find_file_in_path (MODULE_INSTALL_DIR, module_name);
+
    if (file != NULL)
      {
-	(void) import_from_library (symbol_name, symbol_name, deinit_name, file);
+	(void) import_from_library (symbol_name, symbol_name, deinit_name, file, ns, ns_init_name);
 	SLfree (file);
      }
    else
      {
 	/* Maybe the system loader can find it in LD_LIBRARY_PATH */
-	(void) import_from_library (symbol_name, symbol_name, deinit_name, module_name);
+	(void) import_from_library (symbol_name, symbol_name, deinit_name, module_name, ns, ns_init_name);
      }
 }
 
@@ -195,13 +241,17 @@ static void set_import_module_path (char *path)
 
 static char *get_import_module_path (void)
 {
-   if (Module_Path == NULL) return "";
-   return Module_Path;
+   char *path;
+   if (Module_Path != NULL)
+     return Module_Path;
+   if (NULL != (path = getenv (MODULE_PATH_ENV_NAME)))
+     return path;
+   return MODULE_INSTALL_DIR;
 }
 
 static SLang_Intrin_Fun_Type Module_Intrins [] =
 {
-   MAKE_INTRINSIC_S("import", import_module, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("import", import_module, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("set_import_module_path", set_import_module_path, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("get_import_module_path", get_import_module_path, SLANG_STRING_TYPE),
    SLANG_END_INTRIN_FUN_TABLE
