@@ -1,4 +1,4 @@
-/* Copyright (c) 1998, 1999 John E. Davis
+/* Copyright (c) 1998, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -926,7 +926,10 @@ void _SLparse_start (SLang_Load_Type *llt)
 	  save_list = NULL;
 
 	while (Token_List != save_list)
-	  pop_token_list (1);
+	  {
+	     if (-1 == pop_token_list (1))
+	       break;		       /* ??? when would this happen? */
+	  }
      }
 
    free_token (&ctok);
