@@ -1,5 +1,5 @@
 /* ed style regular expressions */
-/* Copyright (c) 1992, 1999, 2001 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  *
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
@@ -459,7 +459,7 @@ unsigned char *SLang_regexp_match(unsigned char *str,
 	c = *(buf + 2);
      }
 
-   while (str < estr)
+   while (1)
      {
 	ctx_buf.open_paren_number = 0;
 	memset (ctx_buf.closed_paren_matches, 0, sizeof(ctx_buf.closed_paren_matches));
@@ -476,6 +476,8 @@ unsigned char *SLang_regexp_match(unsigned char *str,
 	     fixup_beg_end_matches (&ctx_buf, reg, str, epos);
 	     return str;
 	  }
+	if (str >= estr)
+	  break;
 	str++;
      }
    fixup_beg_end_matches (&ctx_buf, reg, NULL, epos);
