@@ -1,5 +1,5 @@
 /* SLang_read_line interface --- uses SLang tty stuff */
-/* Copyright (c) 1992, 1999, 2001 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -324,7 +324,7 @@ static void really_update (SLang_RLine_Info_Type *rli, int new_curs_position)
 
 static void RLupdate (SLang_RLine_Info_Type *rli)
 {
-   int len, dlen, start_len = 0, prompt_len = 0, tw = 0, count;
+   int len, dlen, start_len, prompt_len = 0, tw = 0, count;
    int want_cursor_pos;
    unsigned char *b, chb, *b_point, *p;
    int no_echo;
@@ -362,7 +362,7 @@ static void RLupdate (SLang_RLine_Info_Type *rli)
 	  }
 	tw = rli->tab;
 	b = (unsigned char *) rli->buf;
-	if (count == 1) want_cursor_pos = prompt_len = len;
+	if (count == 1) prompt_len = len;
      }
 
    if (len < rli->edit_width - rli->dhscroll) start_len = 0;
@@ -808,7 +808,7 @@ int SLang_init_readline (SLang_RLine_Info_Type *rli)
 
 SLang_Read_Line_Type *SLang_rline_save_line (SLang_RLine_Info_Type *rli)
 {
-   SLang_Read_Line_Type *rl = NULL;
+   SLang_Read_Line_Type *rl;
    unsigned char *buf;
 
    if ((rli == NULL) || (rli->buf == NULL))
