@@ -1,7 +1,7 @@
 /* -*- mode: C; mode: fold; -*- */
 /* Standard intrinsic functions for S-Lang.  Included here are string
    and array operations */
-/* Copyright (c) 1992, 1999, 2001 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -209,7 +209,7 @@ static void intrin_type_info1 (void)
 
    SLang_free_object (&obj);
 
-   _SLang_push_datatype (type);
+   SLang_push_datatype (type);
 }
 
 static void intrin_type_info (void)
@@ -219,7 +219,7 @@ static void intrin_type_info (void)
    if (-1 == SLang_pop (&obj))
      return;
 
-   _SLang_push_datatype (obj.data_type);
+   SLang_push_datatype (obj.data_type);
    SLang_free_object (&obj);
 }
 
@@ -240,7 +240,7 @@ void _SLstring_intrinsic (void) /*{{{*/
 static void intrin_typecast (void)
 {
    unsigned char to_type;
-   if (0 == _SLang_pop_datatype (&to_type))
+   if (0 == SLang_pop_datatype (&to_type))
      (void) SLclass_typecast (to_type, 0, 1);
 }
 
@@ -320,7 +320,7 @@ static int intrin_integer (char *s)
 
 static void guess_type (char *s)
 {
-   _SLang_push_datatype (SLang_guess_type(s));
+   SLang_push_datatype (SLang_guess_type(s));
 }
 
 static int load_file (char *s)
