@@ -1,6 +1,6 @@
 /* -*- mode: C; mode: fold; -*- */
 /* string manipulation functions for S-Lang. */
-/* Copyright (c) 1992, 1999, 2001 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -1315,7 +1315,7 @@ static int string_match_cmd (char *str, char *pat, int *nptr) /*{{{*/
 
    n--;
    len = strlen(str);
-   if ((n < 0) || ((unsigned int) n >= len))
+   if ((n < 0) || ((unsigned int) n > len))
      {
 	/* SLang_Error = SL_INVALID_PARM; */
 	return 0;
