@@ -1,5 +1,5 @@
 /* Structure type implementation */
-/* Copyright (c) 1998, 1999, 2001 John E. Davis
+/* Copyright (c) 1998, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -7,7 +7,7 @@
  */
 #include "slinclud.h"
 
-#define SL_APP_WANTS_FOREACH
+/* #define SL_APP_WANTS_FOREACH */
 #include "slang.h"
 #include "_slang.h"
 
