@@ -2,7 +2,7 @@
  * the slang SLtt interface.
  */
 
-/* Copyright (c) 1992, 1999, 2001 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -63,7 +63,7 @@ static int make_integer (unsigned char *buf)
  * structure is SLTERMCAP.  In that case, only terminal_names is malloced
  * and the other fields are pointers into it.
  */
-typedef struct
+struct _SLterminfo_Type
 {
 #define SLTERMINFO 1
 #define SLTERMCAP  2
@@ -84,14 +84,14 @@ typedef struct
    unsigned int string_table_size;
    char *string_table;
 
-} Terminfo_Type;
+};
 
-static char *tcap_getstr (char *, Terminfo_Type *);
-static int tcap_getnum (char *, Terminfo_Type *);
-static int tcap_getflag (char *, Terminfo_Type *);
-static int tcap_getent (char *, Terminfo_Type *);
+static char *tcap_getstr (char *, SLterminfo_Type *);
+static int tcap_getnum (char *, SLterminfo_Type *);
+static int tcap_getflag (char *, SLterminfo_Type *);
+static int tcap_getent (char *, SLterminfo_Type *);
 
-static FILE *open_terminfo (char *file, Terminfo_Type *h)
+static FILE *open_terminfo (char *file, SLterminfo_Type *h)
 {
    FILE *fp;
    unsigned char buf[12];
@@ -145,7 +145,7 @@ static unsigned char *read_terminfo_section (FILE *fp, unsigned int size)
    return (unsigned char *) s;
 }
 
-static char *read_terminal_names (FILE *fp, Terminfo_Type *t)
+static char *read_terminal_names (FILE *fp, SLterminfo_Type *t)
 {
    return t->terminal_names = (char *) read_terminfo_section (fp, t->name_section_size);
 }
@@ -157,7 +157,7 @@ static char *read_terminal_names (FILE *fp, Terminfo_Type *t)
  * <term.h>.
  */
 
-static unsigned char *read_boolean_flags (FILE *fp, Terminfo_Type *t)
+static unsigned char *read_boolean_flags (FILE *fp, SLterminfo_Type *t)
 {
    /* Between the boolean section and the number section, a null byte is
     * inserted, if necessary, to ensure that the number section begins on an
@@ -177,7 +177,7 @@ static unsigned char *read_boolean_flags (FILE *fp, Terminfo_Type *t)
  * value represented is -1 or -2, the capability is taken to be missing.
  */
 
-static unsigned char *read_numbers (FILE *fp, Terminfo_Type *t)
+static unsigned char *read_numbers (FILE *fp, SLterminfo_Type *t)
 {
    return t->numbers = read_terminfo_section (fp, 2 * t->num_numbers);
 }
@@ -191,7 +191,7 @@ static unsigned char *read_numbers (FILE *fp, Terminfo_Type *t)
  * (%x) are stored intact in uninterpreted form.
  */
 
-static unsigned char *read_string_offsets (FILE *fp, Terminfo_Type *t)
+static unsigned char *read_string_offsets (FILE *fp, SLterminfo_Type *t)
 {
    return t->string_offsets = (unsigned char *) read_terminfo_section (fp, 2 * t->num_string_offsets);
 }
@@ -201,7 +201,7 @@ static unsigned char *read_string_offsets (FILE *fp, Terminfo_Type *t)
  * null terminated.
  */
 
-static char *read_string_table (FILE *fp, Terminfo_Type *t)
+static char *read_string_table (FILE *fp, SLterminfo_Type *t)
 {
    return t->string_table = (char *) read_terminfo_section (fp, t->string_table_size);
 }
@@ -216,8 +216,7 @@ static char *read_string_table (FILE *fp, Terminfo_Type *t)
  * are implemented by multiple links to the same compiled file.
  */
 
-#define MAX_TI_DIRS 7
-static char *Terminfo_Dirs [MAX_TI_DIRS] =
+static char *Terminfo_Dirs [] =
 {
    NULL, /* $HOME/.terminfo */
    NULL, /* $TERMINFO */
@@ -225,10 +224,14 @@ static char *Terminfo_Dirs [MAX_TI_DIRS] =
    "/usr/lib/terminfo",
    "/usr/share/lib/terminfo",
    "/etc/terminfo",
-   "/usr/local/lib/terminfo"
+   "/usr/local/lib/terminfo",
+#ifdef MISC_TERMINFO_DIRS
+   MISC_TERMINFO_DIRS,
+#endif
+   ""
 };
 
-char *SLtt_tigetent (char *term)
+SLterminfo_Type *_SLtt_tigetent (char *term)
 {
    char *tidir;
    int i;
@@ -236,7 +239,7 @@ char *SLtt_tigetent (char *term)
    char file[1024];
    static char home_ti [1024];
    char *home;
-   Terminfo_Type *ti;
+   SLterminfo_Type *ti;
 
    if (
        (term == NULL)
@@ -246,7 +249,7 @@ char *SLtt_tigetent (char *term)
        )
      return NULL;
 
-   if (NULL == (ti = (Terminfo_Type *) SLmalloc (sizeof (Terminfo_Type))))
+   if (NULL == (ti = (SLterminfo_Type *) SLmalloc (sizeof (SLterminfo_Type))))
      {
 	return NULL;
      }
@@ -260,7 +263,7 @@ char *SLtt_tigetent (char *term)
    else
 #endif
    /* If we are on a termcap based system, use termcap */
-   if (0 == tcap_getent (term, ti)) return (char *) ti;
+   if (0 == tcap_getent (term, ti)) return ti;
 
    if (NULL != (home = getenv ("HOME")))
      {
@@ -272,18 +275,24 @@ char *SLtt_tigetent (char *term)
 
    Terminfo_Dirs[1] = getenv ("TERMINFO");
    i = 0;
-   while (i < MAX_TI_DIRS)
+   while (1)
      {
 	tidir = Terminfo_Dirs[i];
-	if ((tidir != NULL)
-	    && (sizeof (file) > strlen (tidir) + 2 + strlen (term)))
+	if (tidir != NULL)
 	  {
-	     sprintf (file, "%s/%c/%s", tidir, *term, term);
-	     if (NULL != (fp = open_terminfo (file, ti)))
-	       break;
+	     if (*tidir == 0)
+	       break;		       /* last one */
+
+	     if (sizeof (file) > strlen (tidir) + 2 + strlen (term))
+	       {
+		  sprintf (file, "%s/%c/%s", tidir, *term, term);
+		  if (NULL != (fp = open_terminfo (file, ti)))
+		    break;
+	       }
 	  }
 	i++;
      }
+
 #ifdef SLANG_UNTIC
    fp_open_label:
 #endif
@@ -303,7 +312,7 @@ char *SLtt_tigetent (char *term)
 				 /* success */
 				 fclose (fp);
 				 ti->flags = SLTERMINFO;
-				 return (char *) ti;
+				 return ti;
 			      }
 			    SLfree ((char *)ti->string_offsets);
 			 }
@@ -754,7 +763,7 @@ static Tgetstr_Map_Type Tgetstr_Map [] =
    {"", -1		UNTIC_COMMENT(NULL)}
 };
 
-static int compute_cap_offset (char *cap, Terminfo_Type *t, Tgetstr_Map_Type *map, unsigned int max_ofs)
+static int compute_cap_offset (char *cap, SLterminfo_Type *t, Tgetstr_Map_Type *map, unsigned int max_ofs)
 {
    char cha, chb;
 
@@ -773,12 +782,12 @@ static int compute_cap_offset (char *cap, Terminfo_Type *t, Tgetstr_Map_Type *ma
    return -1;
 }
 
-char *SLtt_tigetstr (char *cap, char **pp)
+char *_SLtt_tigetstr (SLterminfo_Type *t, char *cap)
 {
    int offset;
-   Terminfo_Type *t;
 
-   if ((pp == NULL) || (NULL == (t = (Terminfo_Type *) *pp))) return NULL;
+   if (t == NULL)
+     return NULL;
 
    if (t->flags == SLTERMCAP) return tcap_getstr (cap, t);
 
@@ -833,12 +842,12 @@ static Tgetstr_Map_Type Tgetnum_Map[] =
    {"", -1		UNTIC_COMMENT(NULL)}
 };
 
-int SLtt_tigetnum (char *cap, char **pp)
+int _SLtt_tigetnum (SLterminfo_Type *t, char *cap)
 {
    int offset;
-   Terminfo_Type *t;
 
-   if ((pp == NULL) || (NULL == (t = (Terminfo_Type *) *pp))) return -1;
+   if (t == NULL)
+     return -1;
 
    if (t->flags == SLTERMCAP) return tcap_getnum (cap, t);
 
@@ -896,12 +905,11 @@ static Tgetstr_Map_Type Tgetflag_Map[] =
    {"", -1		UNTIC_COMMENT(NULL)}
 };
 
-int SLtt_tigetflag (char *cap, char **pp)
+int _SLtt_tigetflag (SLterminfo_Type *t, char *cap)
 {
    int offset;
-   Terminfo_Type *t;
 
-   if ((pp == NULL) || (NULL == (t = (Terminfo_Type *) *pp))) return -1;
+   if (t == NULL) return -1;
 
    if (t->flags == SLTERMCAP) return tcap_getflag (cap, t);
 
@@ -915,7 +923,7 @@ int SLtt_tigetflag (char *cap, char **pp)
  * variable.  This variable must contain the termcap entry and NOT the file.
  */
 
-static int tcap_getflag (char *cap, Terminfo_Type *t)
+static int tcap_getflag (char *cap, SLterminfo_Type *t)
 {
    char a, b;
    char *f = (char *) t->boolean_flags;
@@ -956,14 +964,14 @@ static char *tcap_get_cap (unsigned char *cap, unsigned char *caps, unsigned int
    return NULL;
 }
 
-static int tcap_getnum (char *cap, Terminfo_Type *t)
+static int tcap_getnum (char *cap, SLterminfo_Type *t)
 {
    cap = tcap_get_cap ((unsigned char *) cap, t->numbers, t->num_numbers);
    if (cap == NULL) return -1;
    return atoi (cap);
 }
 
-static char *tcap_getstr (char *cap, Terminfo_Type *t)
+static char *tcap_getstr (char *cap, SLterminfo_Type *t)
 {
    return tcap_get_cap ((unsigned char *) cap, (unsigned char *) t->string_table, t->string_table_size);
 }
@@ -977,7 +985,7 @@ static int tcap_extract_field (unsigned char *t0)
 }
 
 int SLtt_Try_Termcap = 1;
-static int tcap_getent (char *term, Terminfo_Type *ti)
+static int tcap_getent (char *term, SLterminfo_Type *ti)
 {
    unsigned char *termcap, ch;
    unsigned char *buf, *b;
@@ -1129,3 +1137,28 @@ static int tcap_getent (char *term, Terminfo_Type *ti)
    ti->flags = SLTERMCAP;
    return 0;
 }
+
+
+/* These routines are provided only for backward binary compatability.
+ * They will vanish in V2.x
+ */
+char *SLtt_tigetent (char *s)
+{
+   return (char *) _SLtt_tigetent (s);
+}
+
+extern char *SLtt_tigetstr (char *s, char **p)
+{
+   if (p == NULL)
+     return NULL;
+   return _SLtt_tigetstr ((SLterminfo_Type *) *p, s);
+}
+
+extern int SLtt_tigetnum (char *s, char **p)
+{
+   if (p == NULL)
+     return -1;
+   return _SLtt_tigetnum ((SLterminfo_Type *) *p, s);
+}
+
+
