@@ -1,4 +1,4 @@
-/* Copyright (c) 1998, 1999, 2001 John E. Davis
+/* Copyright (c) 1998, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -842,12 +842,23 @@ static int prep_exists_function (char *line, char comment)
 static int prep_eval_expr (char *expr)
 {
    int ret;
+   char *end;
 
-   if (0 != SLang_load_string (expr))
+   end = strchr (expr, '\n');
+   if (end == NULL)
+     end = expr + strlen (expr);
+   expr = SLmake_nstring (expr, (unsigned int) (end - expr));
+   if (expr == NULL)
      return -1;
-   if (-1 == SLang_pop_integer (&ret))
-     return -1;
-   return (ret != 0);
+	
+   if ((0 != SLang_load_string (expr))
+       || (-1 == SLang_pop_integer (&ret)))
+     ret = -1;
+   else
+     ret = (ret != 0);
+
+   SLfree (expr);
+   return ret;
 }
 
 
@@ -1096,7 +1107,7 @@ int SLang_guess_type (char *t)
 {
    char *p;
    register char ch;
-   int modifier = 0;
+   int modifier;
 
    if (*t == '-') t++;
    p = t;
@@ -1475,7 +1486,7 @@ static int escape_string (unsigned char *s, unsigned char *smax,
 	     if (buf < buf_max) *buf++ = '0';
 	     if (buf < buf_max) *buf++ = '0';
 	     *is_escaped = 1;
-	     return 0;
+	     break; /* return 0; */
 
 	   case '\n':
 	     *buf++ = '\\';
