@@ -1,5 +1,5 @@
 /* Basic type operations for S-Lang */
-/* Copyright (c) 1992, 1996, 2001 John E. Davis
+/* Copyright (c) 1992, 1996, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -12,7 +12,7 @@
 # include <math.h>
 #endif
 
-#define SL_APP_WANTS_FOREACH	       /* for String_Type */
+/* #define SL_APP_WANTS_FOREACH	*/       /* for String_Type */
 #include "slang.h"
 #include "_slang.h"
 
@@ -335,6 +335,16 @@ static int string_to_int (unsigned char a_type, VOID_STAR ap, unsigned int na,
    return 1;
 }
 
+static int string_acopy (SLtype unused, VOID_STAR src_sptr, VOID_STAR dest_sptr)
+{
+   char *s;
+   (void) unused;
+   if (NULL == (s = SLang_create_slstring (*(char **)src_sptr)))
+     return -1;
+   *(char **)dest_sptr = s;
+   return 0;
+}
+
 struct _SLang_Foreach_Context_Type
 {
    char *string;
@@ -865,6 +875,7 @@ int _SLregister_types (void)
 {
    SLang_Class_Type *cl;
 
+#if 1
    /* A good compiler should optimize this code away. */
    if ((sizeof(short) != SIZEOF_SHORT)
        || (sizeof(int) != SIZEOF_INT)
@@ -872,7 +883,7 @@ int _SLregister_types (void)
        || (sizeof(float) != SIZEOF_FLOAT)
        || (sizeof(double) != SIZEOF_DOUBLE))
      SLang_exit_error ("S-Lang Library not built properly.  Fix SIZEOF_* in config.h and recompile");
-
+#endif
    if (-1 == _SLclass_init ())
      return -1;
 
@@ -909,6 +920,7 @@ int _SLregister_types (void)
      return -1;
    (void) SLclass_set_destroy_function (cl, string_destroy);
    (void) SLclass_set_push_function (cl, string_push);
+   (void) SLclass_set_acopy_function (cl, string_acopy);
    cl->cl_foreach_open = string_foreach_open;
    cl->cl_foreach_close = string_foreach_close;
    cl->cl_foreach = string_foreach;
