@@ -1,5 +1,5 @@
 /* slutty.c --- Unix Low level terminal (tty) functions for S-Lang */
-/* Copyright (c) 1992, 1999, 2001 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -240,7 +240,8 @@ int SLang_init_tty (int abort_char, int no_flow_control, int opost)
        || (1 != isatty (SLang_TT_Read_FD)))
      {
 #ifdef O_RDWR
-# ifndef __BEOS__  /* I have been told that BEOS will HANG if passed /dev/tty */
+# if !defined(__BEOS__) && !defined(__APPLE__)
+	/* I have been told that BEOS will HANG if passed /dev/tty */
 	if ((SLang_TT_Read_FD = open("/dev/tty", O_RDWR)) >= 0)
 	  {
 	     TTY_Open = 1;
@@ -286,6 +287,8 @@ int SLang_init_tty (int abort_char, int no_flow_control, int opost)
      }
 
 #ifndef HAVE_TERMIOS_H
+   (void) opost;
+   (void) no_flow_control;
    newtty.s.sg_flags &= ~(ECHO);
    newtty.s.sg_flags &= ~(CRMOD);
    /*   if (Flow_Control == 0) newtty.s.sg_flags &= ~IXON; */
@@ -466,7 +469,12 @@ int _SLsys_input_pending(int tsecs)
    struct timeval wait;
    long usecs, secs;
 
-   if (TTY_Inited == 0) return -1;
+   if ((TTY_Inited == 0)
+       || (SLang_TT_Read_FD < 0))
+     {
+	errno = EBADF;
+	return -1;
+     }
 
    if (tsecs >= 0)
      {
